# LSB Test Execution Framework
# Common X11 test module
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF with some parts
# derived from lsb-autotest written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# Common interface class for X11-oriented tests
    
package X11_common;
use strict;

use Tests::Test_common;
@X11_common::ISA = qw(Test_common);

use Common;

=head ##
Class X11_common Inherits Test_common {
    overrides &check;
}
=cut ##

sub new {
  my $self  = Test_common->new();
  bless($self);
  return $self;
}

sub check_fonts {
    my $self = shift;
    # 100dpi fonts are needed for the X-oriented tests.
    
    # Check 
    my $x_font_dir = capture('grep -e "FontPath" /etc/X11/xorg.conf'
            .' | grep 75dpi | head -n1' );
    $x_font_dir =~ s/^\s+FontPath\s+\"?//;
    $x_font_dir =~ s/\/75dpi.*//;
    chomp($x_font_dir);
    
    if ( !$x_font_dir ) {
        $x_font_dir = capture('cat /etc/X11/fs/config | grep 75dpi'
                .' | head -n1'
            );
        $x_font_dir =~ s/^\s+\"?//;
        $x_font_dir =~ s/\"?,?//;
        $x_font_dir =~ s/\/75dpi.*//;
        chomp($x_font_dir);
    }
    
    if ( $x_font_dir !~ /^\// || !-d $x_font_dir ) {
        debug_inform(
            "Coldn't locate font directory using the /etc/X11/xorg.conf" );
        $x_font_dir = "";
    }

    # Look where the basic fonts could be.
    if ( !$x_font_dir && -d "/usr/X11R6/lib/X11/fonts/misc" ) {
        $x_font_dir = "/usr/X11R6/lib/X11/fonts";
    }
    if ( !$x_font_dir && -d "/usr/share/fonts/X11/misc" ) {
        $x_font_dir = "/usr/share/fonts/X11";
    }
    if ( !$x_font_dir && -d "/usr/share/X11/fonts/misc" ) {
        $x_font_dir = "/usr/share/X11/fonts";
    }
    if ( !$x_font_dir && -d "/usr/share/fonts/misc" ) {
        $x_font_dir = "/usr/share/fonts";
    }
    if ( !$x_font_dir ) {
        complain("Could not find the X11 font directory.");
        return;
    }

    if ( ! -d "$x_font_dir/misc/" ) {
        complain("Could not find the X11 fonts directory\n"
             ."($x_font_dir/misc/)." );
        return;
    }
    if ( ! -d "$x_font_dir/100dpi/" ) {
        complain("Could not find the X11 fonts directory"
             ."\n($x_font_dir/100dpi/)." );
        return;
    }
    if ( ! -d "$x_font_dir/75dpi/" ) {
        complain("Could not find the X11 fonts directory"
             ."\n($x_font_dir/75dpi/)." );
        return;
    }

    unless (-f "$x_font_dir/100dpi/timR24.pcf.gz") {
        complain ("Could not find 100dpi X fonts which are required "
            ."for X tests\n($x_font_dir/100dpi/timR24.pcf.gz),"
            ."\nPlease install them first." );
        return;
    }
}

sub check {
    my $self = shift;

    # Check for additional tools and packages.

    # The X virtual framebuffer server is used for testing.
    unless (does_shell_know ("Xvfb")) {
        complain ("The ".$self->name." test can't be run."
            ." Please install the Xvfb server first."
        );
    }
    
    check_fonts($self);
    
    Test_common::check($self);
}


sub run {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    
    return Test_common::run($self);
}

sub cleanup {
    my $self = shift;
    
    cmd ("killall -q Xvfb");
    
    return Test_common::cleanup($self);
}

1; # return value
