#!/usr/bin/perl -w

# LSB Test Execution Framework
# Get Certified Page Module (tests_cert.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @lsb_versions_list = qw/4.0 3.2 3.1 3.0 2.0/;

my %archs_list = ();

my $error_show = " display: none;";
my @error_msg = ();
my @warn_msg = ();

my %profile_data = (
	"name" => "",
	"organization" => "",
	"email" => "",
	"send_email" => "",
	"architecture" => "ia32",
	"keep_packages" => " checked=\"checked\"",
	"lsb_version" => "3.1",
	"verbose_level" => "1",
	"comments" => ""
);

my $error_text = "";
my $warn_text = "";
my $empty_output = 1;

if (open(FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list 2>/tmp/tests_error.$$ |")) {
	while (<FILE>) {
		$empty_output = 0;
		s/[\r\n]//g;
		if (/^ERROR: (.*)$/i) {
			$error_text .= $1."<br />";
		}
		elsif (/^WARNING: (.*)$/i) {
			$warn_text .= $1."<br />";
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^(EXPIRED:|FILE_SIZE:|MODULE_TIME:)\s+(.*)$/) {
		}
		elsif (/^[^\s#]/) {
			my ($lsb_version, $arch, $mod, $stage, $vers, $files) = split(/\s+/, $_, 6);
			$archs_list{$arch} = 1;
		}
	}
	close(FILE);
	if (-s "/tmp/tests_error.$$" != 0) {
		if (!$empty_output) {
			$error_show = "";
			push @error_msg, "There were some errors while retrieving the list of available test modules!<br />See details in the <a href=\"get.pl?file=$APP_DATA/log/server.log.8888\"><u>server log file</u></a>.";
		}
		print STDERR `cat /tmp/tests_error.$$`;
	}
	unlink("/tmp/tests_error.$$");
	if ($empty_output) {
		$error_show = "";
		push @error_msg, "Cannot get the system information: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
	}
	if ($error_text ne "") {
		$error_show = "";
		push @error_msg, $error_text;
	}
	if ($warn_text ne "") {
		$error_show = "";
		push @warn_msg, $warn_text;
	}
}
else {
	$error_show = "";
	push @error_msg, "Cannot get the system information: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
}

if (!-d "$APP_DATA/profiles") {
	if (!mkdir("$APP_DATA/profiles")) {
		$error_show = "";
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f "$APP_DATA/profiles/~session.$PeerIP.".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f "$APP_DATA/profiles/~session.$PeerIP.$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.$PeerIP.$i";
	if (open(FILE, "> $APP_DATA/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		$error_show = "";
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.$PeerIP.".$_COOKIE{"session_id"};
}

if (open(FILE, "$APP_DATA/profiles/$profile_name")) {
	my @comments = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^NAME:\s+(.*)$/) {
			$profile_data{"name"} = $1;
		}
		elsif (/^ORGANIZATION:\s+(.*)$/) {
			$profile_data{"organization"} = $1;
		}
		elsif (/^EMAIL:\s+(.*)$/) {
			$profile_data{"email"} = $1;
		}
		elsif (/^SEND_EMAIL:\s+(.*)$/) {
			$profile_data{"send_email"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
			$profile_data{"verbose_level"} = $1;
		}
		elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
			$profile_data{"keep_packages"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
	}
	close(FILE);
	$profile_data{"comments"} = join("\n", @comments);
	if ($profile_data{"email"} eq "") {
		$profile_data{"send_email"} = "";
	}
}
elsif (!$create_session_error) {
	$error_show = "";
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if ($_COOKIE{"last_error_text"}) {
	$error_show = "";
	push @error_msg, $_COOKIE{"last_error_text"};
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; path=/;\n";
print "Set-Cookie: last_error_text=; path=/;\n\n";

print_header("Get Certified", "cert");

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

print <<DATA;
<noscript><h1 align="center"><font color="red" size="+1">WARNING! Your browser does not support JavaScript.</font></h1></noscript>
<div id="error_msg_collapsed" style="border: dashed 1px darkred; background-color: red; float: right; cursor: pointer; display: none;" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='none';" title="Expand">
<table border="0" cellpadding="1" cellspacing="1">
  <tr>
    <td align="left" style="white-space: nowrap;"><font color="white"><b>&nbsp;There are some problems!</b></font></td>
    <td width="15px"></td>
    <td width="11px"><img src="images/expand.png" alt="O" width="20" height="20" /></td>
  </tr>
</table>
</div>
<div id="error_msg_area" style="border: none;$error_show">
<table border="0" cellpadding="1" cellspacing="1" width="70%" align="center">
  <tr>
    <th style="border: dashed 1px darkred; background: red; color: white; font-size: large;">
      <table border="0" cellpadding="0" cellspacing="0" align="center">
        <tr>
          <td width="100%">Errors!</td>
          <td><img src="images/collapse.png" alt="_" width="20" height="20" style="cursor: pointer;" title="Collapse" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='none';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='';" /></td>
        </tr>
      </table>
    </th>
  </tr>
  <tr><td align="center" style="border: dashed 1px darkred; border-top: none;">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr><td height="10px" colspan="3"></td></tr>
      <tr><td width="50%"></td><td align="left" style="font-size: medium; white-space: nowrap;">$error_text</td><td width="50%"></td></tr>
      <tr><td height="10px" colspan="3"></td></tr>
    </table>
  </td></tr>
</table>
</div>
<h1>Configure Tests</h1>
DATA

print <<DATA;
<form method="post" action="tests_conf_save.pl" name="configure_test">
<input type="hidden" name="certify" value="1" />
<input type="hidden" name="use_internet" value="on" />
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="action" id="form_action" value="" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
<table border="0" cellpadding="2" cellspacing="2">
  <tr>
    <td width="130px">Your name:</td>
    <td width="170px"><input type="text" name="tester_name" id="tester_name" value="$profile_data{name}" style="width: 150px;" /></td>
    <td width="220px"></td>
  </tr>
  <tr>
    <td>Organization:</td>
    <td><input type="text" name="organization" id="organization" value="$profile_data{organization}" style="width: 150px;" /></td>
    <td></td>
  </tr>
  <tr>
    <td>E-mail:</td>
    <td><input type="text" name="tester_email" value="$profile_data{email}" style="width: 150px;" /></td>
    <td><input type="checkbox" name="send_email_report" id="send_email_report"$profile_data{send_email} /> Send report via E-mail</td>
  </tr>
  <tr>
    <td>Select the architecture:</td>
    <td>
      <select name="architecture" id="arch_select" style="width: 156px;">
DATA

if (scalar(keys(%archs_list)) == 0) {
	print "        <option value=\"$profile_data{architecture}\" selected=\"selected\">$profile_data{architecture}</option>\n";
}
else {
	foreach (sort keys %archs_list) {
		if ($_ eq $profile_data{"architecture"}) {
			print "        <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "        <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
      </select>
    </td>
    <td></td>
  </tr>
  <tr>
    <td colspan="3" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:show_hide_details();">&nbsp;<img src="images/arr_d.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional details</td>
  </tr>
  <tr>
    <td colspan="3" id="additional_data" style="border: dashed 1px gray; border-top: none;">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="2"><input type="checkbox" name="keep_packages" id="keep_packages"$profile_data{keep_packages} /> Keep packages&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#keep_packages" target="_blank">?</a>]</b></td>
        </tr>
        <tr>
          <td width="127px">Verbose Level:</td>
          <td width="393px">
            <select name="verbose_level" style="width: 156px;">
DATA
print "              <option value=\"1\"".(($profile_data{verbose_level} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>";
print "              <option value=\"2\"".(($profile_data{verbose_level} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>";
print <<DATA;
            </select>
          </td>
        </tr>
        <tr>
          <td>LSB Version:</td>
          <td>
            <select name="lsb_version" id="lsbv_select" style="width: 156px;">
DATA

foreach (@lsb_versions_list) {
	if ($_ eq $profile_data{"lsb_version"}) {
		print "              <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "              <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
            </select>
          </td>
        </tr>
        <tr>
          <td colspan="2">Comments:</td>
        </tr>
        <tr>
          <td colspan="2">
            <textarea name="comments" rows="5" cols="54">$profile_data{comments}</textarea>
          </td>
        </tr>
      </table>
    </td>
  </tr>
  <tr>
    <td colspan="3"><br /><hr /></td>
  </tr>
</table>
<br />
<b><a href="#" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');} return false;" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></a></b><br />
<div id="warning" style="display: none;"><br />
<div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 80%; padding: 3px; font-family: monospace;">
WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT
HOLDS IMPORTANT DATA! The tests sometimes put a heavy
stress on the tested computer, and in rare unfortunate
cases they can crash the system or damage the stored data.
They also are a security risk as they pass along the root
password.
</div>
</div>
<br />
<input type="submit" name="run_tests" id="run_tests_button" value="Run tests" style="width: 150px;" onclick="javascript:return do_action('run_tests');" />
    </td>
    <td width="50px">&nbsp;&nbsp;&nbsp;&nbsp;</td>
    <td valign="top" width="50%">
      <h2>Profile Management&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#profile_management" target="_blank">?</a>]</b></h2>
      <table border="0" cellpadding="2" cellspacing="2">
        <tr>
          <td width="80px">Save profile:</td>
          <td width="170px"><input type="text" name="save_profile_name" id="save_profile_name" value="" style="width: 150px;" /></td>
          <td width="90px"><input type="submit" name="save_profile_button" id="save_profile_button" value="Save" style="width: 80px;" onclick="javascript:return do_action('save_profile');" /></td>
          <td width="90px"></td>
        </tr>
DATA

my $found = 0;
my $profiles_list = "";
if (opendir(DIR, "$APP_DATA/profiles")) {
	my @files = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@files) {
		if ($_GET{"profile"} and ($_GET{"profile"} eq $_)) {
			$profiles_list .= "              <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			$profiles_list .= "              <option value=\"$_\">$_</option>\n";
		}
		$found = 1;
	}
	closedir(DIR);
}

if ($found) {
	print <<DATA;
        <tr>
          <td>Load profile:</td>
          <td>
            <select name="load_profile_name" id="load_profile_name" style="width: 156px;">
$profiles_list            </select>
          </td>
          <td><input type="submit" name="load_profile_button" id="load_profile_button" value="Load" style="width: 80px;" onclick="javascript:return do_action('load_profile');" /></td>
          <td><input type="submit" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 80px;" onclick="javascript:return do_action('delete_profile');" /></td>
        </tr>
DATA
}
else {
	print <<DATA;
        <tr>
          <td>Load profile:</td>
          <td>
            <select name="load_profile_name" id="load_profile_name" style="width: 156px;" disabled="disabled">
              <option value="">&lt;No profiles present&gt;</option>
            </select>
          </td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 80px;" disabled="disabled" /></td>
          <td><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 80px;" disabled="disabled" /></td>
        </tr>
DATA
}

print <<DATA;
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
var im1 = new Image;
im1.src = "images/arr_r.png";
var im2 = new Image;
im2.src = "images/expand.png";

function show_hide_details() {
	var elem_div = document.getElementById("additional_data");
	var elem_img = document.getElementById("block_arrow_img");
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == "none");
		elem_div.style.display = to_show ? "" : "none";
		elem_img.setAttribute("src", to_show ? "images/arr_d.png" : "images/arr_r.png");
	}
}

function do_action(name) {
	var elem = document.getElementById("form_action");
	if (!elem) {
		alert("Internal error: Cannot find 'Action' field!\\nPlease, contact the authors.");
		return false;
	}
	if (name == "delete_profile") {
		var elem_tmp = document.getElementById("load_profile_name");
		if (!confirm("Do you really want to delete the profile '" + elem_tmp.value + "'?"))
			return false;
	}
	else if (name == "save_profile") {
		var elem_tmp = document.getElementById("save_profile_name");
		if (!elem_tmp || (elem_tmp.value == "")) {
			elem_tmp = document.getElementById("load_profile_name");
			if (elem_tmp) {
				if (!elem_tmp.value) {
					alert("Please, specify the profile name!");
					return false;
				}
				else if (!confirm("Do you want to overwrite the profile '" + elem_tmp.value + "'?"))
					return false;
			}
			else
				return false;
		}
	}
	else if (name == "run_tests") {
		var elem1 = document.getElementById("organization");
		var elem2 = document.getElementById("tester_name");
		if (!elem1 || (elem1.value == "") || !elem2 || (elem2.value == "")) {
			alert("Please, specify your name and organization!");
			return false;
		}
	}
	elem.value = name;
	return true;
}
// ]]>
</script>
DATA

print_footer();
