#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Page Module (tests_conf.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @lsb_versions_list = qw/4.0 3.2 3.1 3.0 2.0/;

# %lsbv_arch:
# %(lsb_version => %(arch => %(module => %(PresV|AvailV|BetaV|PresF|AvailF|BetaF => version|version|version|@file_names|@file_names|@file_names))));
# %archs_list:
# %(arch => 1);
# %file_size:
# %(file_name => size);
# %mod_time:
# %(module_name => time);

my %lsbv_arch = ();
my %archs_list = ();
my %file_size = ();
my %mod_time = ();
my $ftp_list_expired = 0;

my $error_show = " display: none;";
my @error_msg = ();
my @warn_msg = ();

my %profile_data = (
	"name" => "",
	"organization" => "",
	"email" => "",
	"send_email" => "",
	"architecture" => "ia32",
	"use_internet" => " checked=\"checked\"",
	"keep_packages" => " checked=\"checked\"",
	"short_scen" => "",
	"lsb_version" => "3.1",
	"verbose_level" => "1",
	"comments" => ""
);

my $error_text = "";
my $warn_text = "";
my $empty_output = 1;

if (open(FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list 2>/tmp/tests_error.$$ |")) {
	while (<FILE>) {
		$empty_output = 0;
		s/[\r\n]//g;
		if (/^ERROR: (.*)$/i) {
			$error_text .= $1."<br />";
		}
		elsif (/^WARNING: (.*)$/i) {
			$warn_text .= $1."<br />";
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^EXPIRED:\s+(.*)$/) {
			$ftp_list_expired = 1;
		}
		elsif (/^FILE_SIZE:/) {
			my ($sig, $name, $sz) = split(/\s+/, $_, 3);
			$file_size{$name} = $sz;
		}
		elsif (/^MODULE_TIME:/) {
			my ($sig, $name, $tm) = split(/\s+/, $_, 3);
			$mod_time{$name} = $tm;
		}
		elsif (/^[^\s#]/) {
			my ($lsb_version, $arch, $mod, $stage, $vers, $files) = split(/\s+/, $_, 6);
			if ($stage eq "a") {
				$stage = "AVAIL";
			}
			elsif ($stage eq "b") {
				$stage = "BETA";
			}
			elsif ($stage eq "p") {
				$stage = "PRESENT";
			}
			my @file_names = (($files eq "") ? () : split(/,\s*/, $files));
			$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{$stage."_VERS"} = $vers;
			$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{$stage."_FILES"} = \@file_names;
			$archs_list{$arch} = 1;
		}
	}
	close(FILE);
	if (-s "/tmp/tests_error.$$" != 0) {
		if (!$empty_output) {
			$error_show = "";
			push @error_msg, "There were some errors while retrieving the list of available test modules!<br />See details in the <a href=\"get.pl?file=$APP_DATA/log/server.log.8888\"><u>server log file</u></a>.";
		}
		print STDERR `cat /tmp/tests_error.$$`;
	}
	unlink("/tmp/tests_error.$$");
	if ($empty_output) {
		$error_show = "";
		push @error_msg, "Cannot get the list of available test modules: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
	}
	if ($error_text ne "") {
		$error_show = "";
		push @error_msg, $error_text;
	}
	if ($warn_text ne "") {
		$error_show = "";
		push @warn_msg, $warn_text;
	}
}
else {
	$error_show = "";
	push @error_msg, "Cannot get the list of available test modules: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
}

if (!-d "$APP_DATA/profiles") {
	if (!mkdir("$APP_DATA/profiles")) {
		$error_show = "";
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f "$APP_DATA/profiles/~session.$PeerIP.".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f "$APP_DATA/profiles/~session.$PeerIP.$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.$PeerIP.$i";
	if (open(FILE, "> $APP_DATA/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		$error_show = "";
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.$PeerIP.".$_COOKIE{"session_id"};
}

if (open(FILE, "$APP_DATA/profiles/$profile_name")) {
	my @comments = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^NAME:\s+(.*)$/) {
			$profile_data{"name"} = $1;
		}
		elsif (/^ORGANIZATION:\s+(.*)$/) {
			$profile_data{"organization"} = $1;
		}
		elsif (/^EMAIL:\s+(.*)$/) {
			$profile_data{"email"} = $1;
		}
		elsif (/^SEND_EMAIL:\s+(.*)$/) {
			$profile_data{"send_email"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
			$profile_data{"verbose_level"} = $1;
		}
		elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
			$profile_data{"keep_packages"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^SHORT_SCENARIOS:\s+(.*)$/) {
			$profile_data{"short_scen"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^USE_INTERNET:\s+(.*)$/) {
			$profile_data{"use_internet"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
		elsif (/^[^\s#]/) {
			my ($mod, $stage) = split(/\s+/, $_, 2);
			$profile_data{"modules"}->{$mod} = $stage;
		}
	}
	close(FILE);
	$profile_data{"comments"} = join("\n", @comments);
	if ($profile_data{"email"} eq "") {
		$profile_data{"send_email"} = "";
	}
}
elsif (!$create_session_error) {
	$error_show = "";
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if ($_COOKIE{"last_error_text"}) {
	$error_show = "";
	push @error_msg, $_COOKIE{"last_error_text"};
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; path=/;\n";
print "Set-Cookie: last_error_text=; path=/;\n\n";

print_header("Configure Tests", "conf");

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

print <<DATA;
<noscript><h1 align="center"><font color="red" size="+1">WARNING! Your browser does not support JavaScript.</font></h1></noscript>
<div id="error_msg_collapsed" style="border: dashed 1px darkred; background-color: red; float: right; cursor: pointer; display: none;" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='none';" title="Expand">
<table border="0" cellpadding="1" cellspacing="1">
  <tr>
    <td align="left" style="white-space: nowrap;"><font color="white"><b>&nbsp;There are some problems!</b></font></td>
    <td width="15px"></td>
    <td width="11px"><img src="images/expand.png" alt="O" width="20" height="20" /></td>
  </tr>
</table>
</div>
<div id="error_msg_area" style="border: none;$error_show">
<table border="0" cellpadding="1" cellspacing="1" width="70%" align="center">
  <tr>
    <th style="border: dashed 1px darkred; background: red; color: white; font-size: large;">
      <table border="0" cellpadding="0" cellspacing="0" align="center">
        <tr>
          <td width="100%">Errors!</td>
          <td><img src="images/collapse.png" alt="_" width="20" height="20" style="cursor: pointer;" title="Collapse" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='none';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='';" /></td>
        </tr>
      </table>
    </th>
  </tr>
  <tr><td align="center" style="border: dashed 1px darkred; border-top: none;">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr><td height="10px" colspan="3"></td></tr>
      <tr><td width="50%"></td><td align="left" style="font-size: medium; white-space: nowrap;">$error_text</td><td width="50%"></td></tr>
      <tr><td height="10px" colspan="3"></td></tr>
    </table>
  </td></tr>
</table>
</div>
<h1>Configure Tests</h1>
DATA

print <<DATA;
<form method="post" action="tests_conf_save.pl" name="configure_test">
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="action" id="form_action" value="" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
<table border="0" cellpadding="2" cellspacing="2">
  <tr>
    <td width="130px">Your name:</td>
    <td width="170px"><input type="text" name="tester_name" value="$profile_data{name}" style="width: 150px;" /></td>
    <td width="220px">&nbsp;</td>
  </tr>
  <tr>
    <td>Organization:</td>
    <td><input type="text" name="organization" value="$profile_data{organization}" style="width: 150px;" /></td>
    <td><!--Execution time: <b><span id="estim_time">0&nbsp;min</span></b>--></td>
  </tr>
  <tr>
    <td>E-mail:</td>
    <td><input type="text" name="tester_email" value="$profile_data{email}" style="width: 150px;" /></td>
    <td><input type="checkbox" name="send_email_report" id="send_email_report"$profile_data{send_email} /> Send report via E-mail</td>
  </tr>
  <tr>
    <td>Select the architecture:</td>
    <td>
      <select name="architecture" id="arch_select" style="width: 156px;" onchange="javascript:show_modules();">
DATA

if (scalar(keys(%archs_list)) == 0) {
	print "        <option value=\"$profile_data{architecture}\" selected=\"selected\">$profile_data{architecture}</option>\n";
}
else {
	foreach (sort keys %archs_list) {
		if ($_ eq $profile_data{"architecture"}) {
			print "        <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "        <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
      </select>
    </td>
    <td></td>
  </tr>
  <tr>
    <td colspan="3" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:show_hide_details();">&nbsp;<img src="images/arr_r.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional details</td>
  </tr>
  <tr>
    <td colspan="3" id="additional_data" style="border: dashed 1px gray; border-top: none; display: none;">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="2"><input type="checkbox" name="keep_packages" id="keep_packages"$profile_data{keep_packages} /> Keep packages&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#keep_packages" target="_blank">?</a>]</b></td>
        </tr>
        <tr>
          <td colspan="2"><input type="checkbox" name="short_scen" id="short_scen"$profile_data{short_scen} /> Short scenarios&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#short_scenarios" target="_blank">?</a>]</b></td>
        </tr>
        <tr>
          <td width="127px">Verbose Level:</td>
          <td width="393px">
            <select name="verbose_level" style="width: 156px;">
DATA
print "              <option value=\"1\"".(($profile_data{verbose_level} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>";
print "              <option value=\"2\"".(($profile_data{verbose_level} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>";
print <<DATA;
            </select>
          </td>
        </tr>
        <tr>
          <td>LSB Version:</td>
          <td>
            <select name="lsb_version" id="lsbv_select" style="width: 156px;" onchange="javascript:show_modules();">
DATA

foreach (@lsb_versions_list) {
	if ($_ eq $profile_data{"lsb_version"}) {
		print "              <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "              <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
            </select>
          </td>
        </tr>
        <tr>
          <td colspan="2">Comments:</td>
        </tr>
        <tr>
          <td colspan="2">
            <textarea name="comments" rows="5" cols="54">$profile_data{comments}</textarea>
          </td>
        </tr>
      </table>
    </td>
  </tr>
  <tr>
    <td colspan="3" valign="bottom" height="15"><hr /></td>
  </tr>
  <tr>
    <td valign="top" colspan="2">
      <table><tr>
        <td valign="top"><input type="checkbox" name="use_internet" id="use_internet"$profile_data{use_internet} onclick="javascript:update_elements_state();" ondblclick="javascript:setTimeout('update_elements_state()', 500);" /></td>
        <td>Use Internet connection for downloading the tests<br />(download size: <b><span id="download_sz">0&nbsp;Kbytes</span></b>)</td>
      </tr></table>
    </td>
DATA

my $exp_style = "";
my $exp_warning = "";
my $refresh_button_name = ((scalar(keys(%archs_list)) == 0) ? "Get List" : "Refresh List");

if (scalar(keys(%archs_list)) == 0) {
	$exp_style = " color: red; font-weight: bold;";
	$exp_warning = "  <tr>\n    <td colspan=\"3\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">The list was not obtained yet! Press \"$refresh_button_name\" button.</font></b></td>\n  </tr>\n";
}
elsif ($ftp_list_expired) {
	$exp_style = " color: red; font-weight: bold;";
	$exp_warning = "  <tr>\n    <td colspan=\"3\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">The list may be outdated! Press \"$refresh_button_name\" button.</font></b></td>\n  </tr>\n";
}

if (scalar(keys(%archs_list)) == 0) {
	print "    <td align=\"right\" valign=\"bottom\"><input type=\"submit\" name=\"refresh_list\" id=\"refresh_list_button\" value=\"Get List\" style=\"width: 100px; color: red; font-weight: bold;\" title=\"Get the list of available tests from the Internet\" onclick=\"javascript:return do_action('refresh_list');\" /></td>";
}
else {
	print "    <td align=\"right\" valign=\"bottom\"><input type=\"submit\" name=\"refresh_list\" id=\"refresh_list_button\" value=\"Refresh List\" style=\"width: 100px;$exp_style\" title=\"Refresh the list of available tests from the Internet\" onclick=\"javascript:return do_action('refresh_list');\" /></td>";
}

print <<DATA;
  </tr>
  <tr>
    <td colspan="3"></td>
  </tr>
$exp_warning</table>
DATA

my $lsb_v;

sub print_test_suite_row($$$$) {
	my ($bkgd, $arch, $mod, $lsb_v) = @_;
	my $x1 = $lsbv_arch{$lsb_v};
	my $x2 = $x1->{$arch};
	my $x3 = $x2->{$mod};
	my $stage = $lsbv_arch{$lsb_v}->{$arch}->{$mod};
	my $mod_pp = $mod;
	$mod_pp =~ s/\+/p/g;
	print "  <tr class=\"list_item$bkgd\">\n    <td><input type=\"checkbox\" name=\"mod-$arch-$lsb_v-$mod_pp\" id=\"mod-$arch-$lsb_v-$mod_pp\" onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$mod</td>\n";
	my $statep = " disabled=\"disabled\"".($stage->{"PRESENT_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $statea = " disabled=\"disabled\"".($stage->{"AVAIL_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $stateb = " disabled=\"disabled\"".($stage->{"BETA_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $idp = $stage->{"PRESENT_VERS"} ? "-p" : "-p-";
	my $ida = $stage->{"AVAIL_VERS"} ? "-a" : "-a-";
	my $idb = $stage->{"BETA_VERS"} ? "-b" : "-b-";
	if ($stage->{"PRESENT_VERS"}) {
		$statep = " checked=\"checked\"".$statep;
	}
	elsif ($stage->{"AVAIL_VERS"}) {
		$statea = " checked=\"checked\"".$statea;
	}
	elsif ($stage->{"BETA_VERS"}) {
		$stateb = " checked=\"checked\"".$stateb;
	}
	print "    <td><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$idp\" value=\"p\"$statep /> ".($stage->{"PRESENT_VERS"} or "")."</td>\n";
	print "    <td><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$ida\" value=\"a\"$statea /> ".($stage->{"AVAIL_VERS"} or "")."</td>\n";
	print "    <td><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$idb\" value=\"b\"$stateb /> ".($stage->{"BETA_VERS"} or "")."</td>\n  </tr>\n";
}

foreach $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %{$lsbv_arch{$lsb_v}}) {
		if (($arch eq $profile_data{"architecture"}) and ($lsb_v eq $profile_data{"lsb_version"})) {
			print "<div id=\"arch-$arch-$lsb_v\">\n";
		}
		else {
			print "<div id=\"arch-$arch-$lsb_v\" style=\"display: none;\">\n";
		}
		my $count;
		my %listed = ();
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			$listed{$mod} = 1;
		}
		print <<DATA;
<table border="0" cellpadding="0" cellspacing="0">
  <tr class="list_head">
    <td width="170px"><input type="checkbox" name="check-all-$arch-$lsb_v" id="check-all-$arch-$lsb_v" onclick="javascript:un_check_all('all');" /></td>
    <td width="120px"><b>Localhost</b></td>
    <td width="120px"><b>FTP</b></td>
    <td width="120px"><b>FTP (beta)</b></td>
  </tr>
  <tr class="list_cat">
    <td colspan="4"><input type="checkbox" name="check-cert-$arch-$lsb_v" id="check-cert-$arch-$lsb_v" onclick="javascript:un_check_all('cert');" />&nbsp;<b>Certification Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#cert" target="_blank">?</a>]</b></td>
  </tr>
  <tr class="list_subcat">
    <td colspan="4"><input type="checkbox" name="check-cmdtests-$arch-$lsb_v" id="check-cmdtests-$arch-$lsb_v" onclick="javascript:un_check_all('cmdtests');" />&nbsp;&nbsp;&nbsp;<b>Command Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#cmdtests" target="_blank">?</a>]</b></td>
  </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/cmdchk/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
  <tr class="list_subcat">
    <td colspan="4"><input type="checkbox" name="check-sitests-$arch-$lsb_v" id="check-sitests-$arch-$lsb_v" onclick="javascript:un_check_all('sitests');" />&nbsp;&nbsp;&nbsp;<b>Static Interface Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#sitests" target="_blank">?</a>]</b></td>
  </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/libchk/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
  <tr class="list_subcat">
    <td colspan="4"><input type="checkbox" name="check-ritests-$arch-$lsb_v" id="check-ritests-$arch-$lsb_v" onclick="javascript:un_check_all('ritests');" />&nbsp;&nbsp;&nbsp;<b>Runtime Interface Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#ritests" target="_blank">?</a>]</b></td>
  </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/desktop libstdc++ runtime vsw4/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
  <tr class="list_subcat">
    <td colspan="4"><input type="checkbox" name="check-abtests-$arch-$lsb_v" id="check-abtests-$arch-$lsb_v" onclick="javascript:un_check_all('abtests');" />&nbsp;&nbsp;&nbsp;<b>Automated Application Battery Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#abtests" target="_blank">?</a>]</b></td>
  </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/apache expect groff lynx python raptor rsync samba tcl xpdf/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
  <tr class="list_cat">
    <td colspan="4"><input type="checkbox" name="check-unoff-$arch-$lsb_v" id="check-unoff-$arch-$lsb_v" onclick="javascript:un_check_all('unoff');" />&nbsp;<b>Unofficial Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#unoff" target="_blank">?</a>]</b></td>
  </tr>
  <tr class="list_subcat">
    <td colspan="4"><input type="checkbox" name="check-uritests-$arch-$lsb_v" id="check-uritests-$arch-$lsb_v" onclick="javascript:un_check_all('uritests');" />&nbsp;&nbsp;&nbsp;<b>Runtime Interface Tests&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#uritests" target="_blank">?</a>]</b></td>
  </tr>
DATA
		$count = 0;
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}
		print "</table>\n</div>\n";
	}
}

if (scalar(keys(%archs_list)) != 0) {
	print <<DATA;
<br />
<b><a href="#" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');} return false;" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></a></b><br />
<div id="warning" style="display: none;"><br />
<div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 80%; padding: 3px; font-family: monospace;">
WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT
HOLDS IMPORTANT DATA! The tests sometimes put a heavy
stress on the tested computer, and in rare unfortunate
cases they can crash the system or damage the stored data.
They also are a security risk as they pass along the root
password.
</div>
</div>
DATA
}

print <<DATA;
<br />
<input type="submit" name="run_tests" id="run_tests_button" value="Run selected tests" style="width: 150px;" onclick="javascript:return do_action('run_tests');" />
    </td>
    <td width="50px">&nbsp;&nbsp;&nbsp;&nbsp;</td>
    <td valign="top" width="50%">
      <h2>Profile Management&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#profile_management" target="_blank">?</a>]</b></h2>
      <table border="0" cellpadding="2" cellspacing="2">
        <tr>
          <td width="80px">Save profile:</td>
          <td width="170px"><input type="text" name="save_profile_name" id="save_profile_name" value="" style="width: 150px;" /></td>
          <td width="90px"><input type="submit" name="save_profile_button" id="save_profile_button" value="Save" style="width: 80px;" onclick="javascript:return do_action('save_profile');" /></td>
          <td width="90px"></td>
        </tr>
DATA

my $found = 0;
my $profiles_list = "";
if (opendir(DIR, "$APP_DATA/profiles")) {
	my @files = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@files) {
		if ($_GET{"profile"} and ($_GET{"profile"} eq $_)) {
			$profiles_list .= "              <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			$profiles_list .= "              <option value=\"$_\">$_</option>\n";
		}
		$found = 1;
	}
	closedir(DIR);
}

if ($found) {
	print <<DATA;
        <tr>
          <td>Load profile:</td>
          <td>
            <select name="load_profile_name" id="load_profile_name" style="width: 156px;">
$profiles_list            </select>
          </td>
          <td><input type="submit" name="load_profile_button" id="load_profile_button" value="Load" style="width: 80px;" onclick="javascript:return do_action('load_profile');" /></td>
          <td><input type="submit" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 80px;" onclick="javascript:return do_action('delete_profile');" /></td>
        </tr>
DATA
}
else {
	print <<DATA;
        <tr>
          <td>Load profile:</td>
          <td>
            <select name="load_profile_name" id="load_profile_name" style="width: 156px;" disabled="disabled">
              <option value="">&lt;No profiles present&gt;</option>
            </select>
          </td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 80px;" disabled="disabled" /></td>
          <td><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 80px;" disabled="disabled" /></td>
        </tr>
DATA
}

my $current_lsb = $profile_data{"lsb_version"};
my $current_arch = $profile_data{"architecture"};

print <<DATA;
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
var im1 = new Image;
im1.src = "images/arr_d.png";
var im2 = new Image;
im2.src = "images/expand.png";
var arch_mod = new Array();
var file_size = new Array();
var mod_time = new Array();
var current_arch = "$current_arch-$current_lsb";
DATA

foreach my $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %archs_list) {
		my $mods = $lsbv_arch{$lsb_v}->{$arch};
		print "arch_mod[\"$arch-$lsb_v\"] = new Array();\n";
		foreach my $mod (sort keys %$mods) {
			my $stage = $mods->{$mod};
			my $mod_pp = $mod;
			$mod_pp =~ s/\+/p/g;
			print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"] = new Array();\n";
			if ($stage->{"PRESENT_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"PRESENT_VERS\"] = \"".$stage->{"PRESENT_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"PRESENT_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"PRESENT_FILES"}})."\");\n";
			}
			if ($stage->{"AVAIL_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"AVAIL_VERS\"] = \"".$stage->{"AVAIL_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"AVAIL_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"AVAIL_FILES"}})."\");\n";
			}
			if ($stage->{"BETA_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"BETA_VERS\"] = \"".$stage->{"BETA_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"BETA_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"BETA_FILES"}})."\");\n";
			}
		}
	}
}

foreach my $fname (sort keys %file_size) {
	print "file_size[\"$fname\"] = \"".$file_size{$fname}."\";\n";
}
print "file_size[\"\"] = \"0\";\n";

foreach my $mname (sort keys %mod_time) {
	print "mod_time[\"$mname\"] = \"".$mod_time{$mname}."\";\n";
}

print "var arch_list = new Array(\"".join("\", \"", keys %archs_list)."\");\n";
print "var lsbv_list = new Array(\"".join("\", \"", @lsb_versions_list)."\");\n";

my %mod_list = ();
foreach $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %{$lsbv_arch{$lsb_v}}) {
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			$mod =~ s/\+/p/g;
			$mod_list{$mod} = 1;
		}
	}
}

print "var mod_list = new Array(\"".join("\", \"", sort keys %mod_list)."\");\n\n";

print <<DATA;
function mergeArrays(arr1, arr2) {
	var res = arr1;
	var i, j, k = res.length;
	var found;
	for (i=0; i<arr2.length; ++i) {
		found = 0;
		for (j=0; j<arr1.length; ++j) {
			if (arr1[j] == arr2[i]) {
				found = 1;
				break;
			}
		}
		if (!found)
			res[k++] = arr2[i];
	}
	return res;
}

function update_elements_state() {
	var elem;
	var checked;
	var i, j;
	var files = new Array();
	var total_sz = 0;
	var total_time = 0;
	var total_num = 0;

	// Enable/disable "Refresh List" button (as it needs Internet connection)
	elem = document.getElementById("use_internet");
	var checked_internet = elem ? elem.checked : 0;
	elem = document.getElementById("refresh_list_button");
	if (elem)
		elem.disabled = !checked_internet;

	// for each module...
	for (i=0; i<mod_list.length; ++i) {
		var elemp, elema, elemb;
		var enablep, enablea, enableb;
		// Find the module's checkbox ...
		elem = document.getElementById("mod-" + current_arch + "-" + mod_list[i]);
		if (!elem)
			continue;
		checked = elem.checked;
		// ... and radio-button of stages (present/available/beta).
		elemp = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-p");
		elema = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-a");
		elemb = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-b");
		if (checked_internet) {
			// If Internet is possible, then no problems.
			enablep = true;
			enablea = true;
			enableb = true;
		}
		else {
			// Else we find out if the Internet connection is needed for downloading anything
			// for each of the stage-variants (enable[p|a|b]==1 if there is nothing to download).
			var tmp_files;
			if (elemp) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["PRESENT_FILES"];
				enablep = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enablep = false;
						break;
					}
				}
			}
			else
				enablep = false;
			if (elema) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["AVAIL_FILES"];
				enablea = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enablea = false;
						break;
					}
				}
			}
			else
				enablea = false;
			if (elemb) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["BETA_FILES"];
				enableb = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enableb = false;
						break;
					}
				}
			}
			else
				enableb = false;
		}
		if (enablep || enablea || enableb) {
			// If at least one variant is permitted, allow the module, ...
			elem.disabled = false;
			// ... and the stage-variants, if needed.
			enablep = enablep && checked;
			enablea = enablea && checked;
			enableb = enableb && checked;
			if (elemp)
				elemp.disabled = !enablep;
			if (elema)
				elema.disabled = !enablea;
			if (elemb)
				elemb.disabled = !enableb;
			if (checked) {
				// Change stage to possible, if the selected one is prohibited.
				var preferrable;
				if (elemp && !elemp.disabled) {
					preferrable = elemp;
				}
				else if (elema && !elema.disabled) {
					preferrable = elema;
				}
				else if (elemb && !elemb.disabled) {
					preferrable = elemb;
				}
				if (((elemp && elemp.checked && elemp.disabled) ||
					 (elema && elema.checked && elema.disabled) ||
					 (elemb && elemb.checked && elemb.disabled)) && preferrable)
					preferrable.checked = true;
				// Now add the files needed for downloading to the global list.
				if (elemp && elemp.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["PRESENT_FILES"]);
				}
				if (elema && elema.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["AVAIL_FILES"]);
				}
				if (elemb && elemb.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["BETA_FILES"]);
				}
				++total_num;
				//total_time += parseInt(mod_time[mod_list[i]]);
			}
		}
		else {
			// If no variant is possible, turn the whole module off.
			elem.disabled = true;
			if (elemp)
				elemp.disabled = true;
			if (elema)
				elema.disabled = true;
			if (elemb)
				elemb.disabled = true;
		}
	}
	// Summarize the size of files to download...
	for (i=0; i<files.length; ++i) {
		total_sz += parseInt(file_size[files[i]]);
	}
	// and display this size together with the total time of running the tests.
	document.getElementById("download_sz").innerHTML = Math.round(total_sz / 1024) + "&nbsp;Kbytes";
	//document.getElementById("estim_time").innerHTML =  Math.round(total_time / 60) + "&nbsp;min ";
	elem = document.getElementById("run_tests_button");
	if (elem)
		elem.disabled = (total_num == 0);
}

var mod_cat_list = new Array();
mod_cat_list["all"] = new Array("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf", "xts5");
mod_cat_list["cert"] = new Array("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
mod_cat_list["cmdtests"] = new Array("cmdchk");
mod_cat_list["sitests"] = new Array("libchk");
mod_cat_list["ritests"] = new Array("desktop", "libstdcpp", "runtime", "vsw4");
mod_cat_list["abtests"] = new Array("apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
mod_cat_list["unoff"] = new Array("xts5");
mod_cat_list["uritests"] = new Array("xts5");

var check_list = new Array();
check_list["all"] = new Array("cert", "cmdtests", "sitests", "ritests", "abtests", "unoff", "uritests");
check_list["cert"] = new Array("cmdtests", "sitests", "ritests", "abtests");
check_list["cmdtests"] = new Array();
check_list["sitests"] = new Array();
check_list["ritests"] = new Array();
check_list["abtests"] = new Array();
check_list["unoff"] = new Array("uritests");
check_list["uritests"] = new Array();

function un_check_all(category) {
	var elem = document.getElementById("check-" + category + "-" + current_arch);
	if (elem) {
		var checked = elem.checked;
		for (var i=0; i<mod_cat_list[category].length; ++i) {
			elem = document.getElementById("mod-" + current_arch + "-" + mod_cat_list[category][i]);
			if (elem)
				elem.checked = checked;
		}
		for (var i=0; i<check_list[category].length; ++i) {
			elem = document.getElementById("check-" + check_list[category][i] + "-" + current_arch);
			if (elem)
				elem.checked = checked;
		}
		update_elements_state();
	}
}

function show_modules() {
	var i, j;
	var elem_a = document.getElementById("arch_select");
	var elem_l = document.getElementById("lsbv_select");
	if (!elem_a || !elem_l)
		return;
	var arch_lsb = elem_a.value + "-" + elem_l.value;

	for (i=0; i<lsbv_list.length; i++) {
		for (j=0; j<arch_list.length; j++) {
			var tmp = arch_list[j] + "-" + lsbv_list[i];
			var elem = document.getElementById("arch-" + tmp);
			if (elem) {
				if (tmp == arch_lsb) {
					elem.style.display = "";
					current_arch = tmp;
				}
				else {
					elem.style.display = "none";
				}
			}
		}
	}
	update_elements_state();
}

function show_hide_details() {
	var elem_div = document.getElementById("additional_data");
	var elem_img = document.getElementById("block_arrow_img");
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == "none");
		elem_div.style.display = to_show ? "" : "none";
		elem_img.setAttribute("src", to_show ? "images/arr_d.png" : "images/arr_r.png");
	}
}

function do_action(name) {
	update_elements_state();
	var elem = document.getElementById("form_action");
	if (!elem) {
		alert("Internal error: Cannot find 'Action' field!\\nPlease, contact the authors.");
		return false;
	}
	if (name == "delete_profile") {
		var elem_tmp = document.getElementById("load_profile_name");
		if (!confirm("Do you really want to delete the profile '" + elem_tmp.value + "'?"))
			return false;
	}
	else if (name == "save_profile") {
		var elem_tmp = document.getElementById("save_profile_name");
		if (!elem_tmp || (elem_tmp.value == "")) {
			elem_tmp = document.getElementById("load_profile_name");
			if (elem_tmp) {
				if (!elem_tmp.value) {
					alert("Please, specify the profile name!");
					return false;
				}
				else if (!confirm("Do you want to overwrite the profile '" + elem_tmp.value + "'?"))
					return false;
			}
			else
				return false;
		}
	}
	elem.value = name;
	return true;
}

function check_profile_modules() {
    var to_check = new Array();
    var to_check_cat = new Array();
DATA

my %mod_cat_list = ();
@{$mod_cat_list{"all"}} = ("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf", "xts5");
@{$mod_cat_list{"cert"}} = ("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
@{$mod_cat_list{"cmdtests"}} = ("cmdchk");
@{$mod_cat_list{"sitests"}} = ("libchk");
@{$mod_cat_list{"ritests"}} = ("desktop", "libstdcpp", "runtime", "vsw4");
@{$mod_cat_list{"abtests"}} = ("apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
@{$mod_cat_list{"unoff"}} = ("xts5");
@{$mod_cat_list{"uritests"}} = ("xts5");

foreach my $mod (keys %{$profile_data{"modules"}}) {
	my $stage = $profile_data{"modules"}->{$mod};
	print "\tto_check[\"$current_arch-$current_lsb-$mod\"] = \"$stage\";\n"
}

foreach my $cat (keys %mod_cat_list) {
	my $full = 1;
	foreach (@{$mod_cat_list{$cat}}) {
		if (!($profile_data{"modules"}->{$_})) {
			$full = 0;
			last;
		}
	}
	if ($full) {
		print "\tto_check_cat[\"$cat\"] = 1;\n"
	}
}

print <<DATA;
	for (var mod in to_check) {
		var elem = document.getElementById("mod-" + mod);
		if (elem) {
			elem.checked = 1;
		}
		elem = document.getElementById("modstage-" + mod + "-" + to_check[mod]);
		if (elem) {
			elem.checked = 1;
		}
    }
	for (var cat in to_check_cat) {
		var elem = document.getElementById("check-" + cat + "-$current_arch-$current_lsb");
		if (elem) {
			elem.checked = 1;
		}
	}
}

check_profile_modules();
update_elements_state();
// ]]>
</script>
DATA

print_footer();
